
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%           PGD Deblending           %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%         Test with Real Data        %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%           Gulf of Mexico           %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% -------------------------------------------------------------------------
% 
% This code shows how to use Projected Gradienet Descent (PGD) for 
% deblending with two different projections: 
% 
%   SSA :Singular Spectrum Analysis Filtering in windows
% R-SSA :Robust Singular Spectrum Analysis Filtering in windows
%
% Rongzhin Lin and M D Sacchi
%
% rongzhi@ualberta.ca 
% msacchi@ualberta.ca
% 
% SAIG 2020 
% Deparment of Physics
% University of Alberta 
% 
% -------------------------------------------------------------------------

 clc;
 clear all; 
 close all;

% Set path for demo  

 set_up;

% Load real data from Gulf of Mexico 

   disp('Load data')

   load gom_data.mat;
   dt = 0.004;

   D0 = DD0(1:end,91,:);

   [nt,nr,ns] = size(D0);

% Blend data to simulate simultaenous source adquisition 

   rand('state',3); 
   nsbs = 2;               % Number of blended records 
   nbs = ns/nsbs;          % Number of shots per blended record
   a = 50;
   b = floor(nt/3);
   ntl = nt + a + b;       % Max time of blended record 

   itimes  = a + floor( rand(nbs,nsbs)*(b-1))+1;     % Ramdon firing times 
 
   Param_blend.nt = nt;
   Param_blend.ntl = ntl;
   Param_blend.nbs = nbs;
   Param_blend.nsbs = nsbs;
   Param_blend.nr = nr;
   Param_blend.ns = ns;
   Param_blend.itimes = itimes;

% Blended data is b

    [b]  = blendingOP(D0,Param_blend,1);

% Pseudo-deblended data is D_pseudo 

    [D_pseudo] = blendingOP(b,Param_blend,-1);


% Initialization with zeros: D1 is the solution via PGD with SSA projection
%                            D2 is the solution via PGD with R-SSA projection 

    D1 = zeros(size(D_pseudo));
    D2 = zeros(size(D_pseudo));

% R-SSA parameters

   Param_RSSA.r = 10;            % Desired rank
   Param_RSSA.iters = 500;       % Number of iterations of BFGD 
   Param_RSSA.BFGD_tol = 0.01;   % Tolerance parameter to terminate iters of BFGD 
   Param_RSSA.alpha = 2;         % Robustness (too small hurts the signal, Tukey's parameter)
   Param_RSSA.mu = 0.1;          % Regularization parameter for BFFD 
   Param_RSSA.dt = dt;   
   Param_RSSA.fmin = 1.0;   
   Param_RSSA.fmax = 75.0;

% SSA parameters

   Param_SSA.k = 10;             % Desired rank
   Param_SSA.Op = @SVDSSAOp;     % SVD-based rank-reduction
   Param_SSA.dt = dt;           
   Param_SSA.fmin = 1.0;   
   Param_SSA.fmax = 75.0;

% Parameters for Patching and UnPatching. 
% This are the parameters for windowing the data prior to applying SSA or R-SSA

   Param_win.dt = dt;
   Param_win.it_WL = 100;
   Param_win.it_WO = 20;
   Param_win.ix1_WL =100;
   Param_win.ix1_WO = 10;
   Param_win.x1min = 1;
   Param_win.x1max = ns;

% Extract max eig of B'B where B is the blending operator 
% Notice that max_eig is equal to Blending Factor 

  max_eig = power_method(randn(size(D0)),@blendingOP,Param_blend)

  iter1 = 10;  % Number of iterations for PGD with the non-robust SSA method
  iter2 = 10;  % Number of iterations for PGD with the robust SSA method (BFGD)

% Allocate vectors so save SNR of reconstruction and Misfit (how well
% solution can reproduce the blended records)

  SNR1 = [];
  SNR2 = [];
  MIS1 = [];
  MIS2 = [];

% -------------------------------------
% PGD with SSA (non-robust projection) 
% -------------------------------------

% Initial Step lenght for PGD 

  lambda0 = 2./ceil(max_eig);

  disp('Start PGD with SSA projection');

 for ij = 1: iter1

   b1 = blendingOP(D1, Param_blend, 1); 
   [~, SNR] = quality(D1,D0);
   [RE,  ~] = quality(b1,b  );
   SNR1(ij) = SNR; 
   MIS1(ij) = RE; 

   lambda = lambda0/sqrt(ij);

   [Dtemp1] = blendingOP(b1-b,  Param_blend, -1);
   z1 = D1 - lambda.*(Dtemp1);

    for ir=1:nr

	[Patches,Minval,Maxval,npatch]=SeisPatchlrz(squeeze(z1(:,ir,:)),Param_win); 

         if (ir==1 & ij ==1); disp(['Number of patches to process = ' num2str(npatch)]);end;

        for ip = 1:npatch
            FXPatches(:,:,ip) = FXProcess(squeeze(Patches(:,:,ip)),@SVDSSAOp,Param_SSA);
        end    

        D1(:,ir,:) = SeisUnPatchlrz(FXPatches,nt,Minval,Maxval,Param_win);

     end
 end

% --------------------------------------------
% PGD with Robust SSA (This method uses BFGD)
% --------------------------------------------

 disp('Start PGD with R-SSA projection');

 b0 = 2.;

 for ij = 1:iter2

   b2 = blendingOP(D2, Param_blend, 1); 
   [~, SNR] = quality(D2,D0);
   [RE,  ~] = quality(b2,b );
   SNR2(ij) = SNR; 
   MIS2(ij) = RE; 

   lambda = lambda0/sqrt(ij);

   [Dtemp2] = blendingOP(b2-b,Param_blend,-1);
   z2 = D2 - lambda.*(Dtemp2);

    Param_RSSA.alpha = b0 + 0.4*(ij-1);   %Increase Tukey's parameter 

    for ir=1:nr

      [Patches,Minval,Maxval,npatch]=SeisPatchlrz(squeeze(z2(:,ir,:)),Param_win); 

        if (ir==1 & ij ==1); disp(['Number of patches to process = ' num2str(npatch)]);end;

        for ip = 1:npatch
          FXPatches(:,:,ip) = FXProcess(squeeze(Patches(:,:,ip)),@BFGDSSAOp,Param_RSSA);
        end

      D2(:,ir,:) = SeisUnPatchlrz(FXPatches,nt,Minval,Maxval,Param_win);

    end

 end

% -----------------------------------------------------------------
% -------------------- Diplay results -----------------------------
% -----------------------------------------------------------------

% Plot data

 figure(1);
 set(gcf, 'Position',  [100,200,2000, 800])
 clip = 0.5*max(D_pseudo(:));
 A = squeeze(D0); 
 B = squeeze(D_pseudo); 
 C = squeeze(D1);      
 D = squeeze(D2);      
 imagesc([A,B,C,D],clip*[-1,1]); 
 colormap(gray); 
 text(1,  -20, 'True',                     'FontSize', 26);
 text(250,-20, 'Pseudo-deblended',         'FontSize', 26); 
 text(500,-20, 'PGD Inversion with SSA',   'FontSize', 26); 
 text(750,-20, 'PGD Inversion with R-SSA', 'FontSize', 26); 
 set(gca,'FontSize',24)
 saveas(1,'results.eps','epsc');

% Quality of reconstruction (SNR [dB]) vs iteration 

 figure(2)
 k2 = 1:1:iter2;
 k1 = 1:1:iter1;
 set(gcf, 'Position',  [100,200,600,600])
 plot(k1,SNR1,'r-',k2,SNR2,'b-','LineWidth',2)
 grid 
 legend('SSA Projection', 'Robust SSA Projection','FontSize',20);
 set(gca,'FontSize',24)
 axis([0.1,max(iter1),0,12]);
 xlabel('PGD Iteration'); 
 ylabel('SNR (dB)');
 saveas(2,'convergence.eps','epsc');

% Average Spectra of results

 figure(3);
 [PA,f] = smooth_spectrum(A,dt,10,'db'); 
 [PB,f] = smooth_spectrum(B,dt,10,'db'); 
 [PC,f] = smooth_spectrum(C,dt,10,'db'); 
 [PD,f] = smooth_spectrum(D,dt,10,'db'); 

 set(gcf, 'Position',  [100,200,600,600])
 plot(f,PA,'k-', f,PB,'g-', f, PC,'r-', f, PD,'b-','LineWidth',2)
 grid 
 legend('True', 'Pseudo','SSA Projection', 'Robust SSA Projection','FontSize',20);
 set(gca,'FontSize',24)
 axis([0.,100.,-30., .4]);
 xlabel('Frequency (Hz)');
 ylabel('Power Spectrum (dB)');
 saveas(3,'spectrum.eps','epsc');

% Misfit of PGD  vs iteration to measure convergence

 figure(4);
 set(gcf, 'Position',  [100,200,600,600])
 plot(k1,MIS1,'r-',k2,MIS2,'b-','LineWidth',2)
 grid 
 legend('SSA Projection', 'Robust SSA Projection','FontSize',20);
 set(gca,'FontSize',24)
 axis([0.,max(iter2),0,100]);
 xlabel('PGD Iteration');
 ylabel('Relative MSE X 100');
 saveas(4,'misfit.eps','epsc');

