function [Xhat] = BFGD(f, f_grad, params)
% ==================================================================================
%  Bifactored Gradient Descent algorithm for low-rank matrix solutions.
% 
% References:
%
% [1] "Finding low-rank solutions to matrix problems, efficiently and provably",
% by Dohyung Park, Anastasios Kyrillidis and, Constantine Caramanis, and 
% Sujay Sanghavi.
% ==================================================================================
%
% INPUT ARGUMENTS:
% f                         Operator that computes the loss function
% f_grad                    Operator that computes gradient of loss function
% params                    Structure of parameters. See main & BFGDSSAOp file for
%                           configuration
% ==================================================================================
% 
% OUTPUT ARGUMENTS:
% Xhat                     m x n recovered rank-r matrix.
% ==================================================================================
% 09/01/2016, by Anastasios Kyrillidis. anastasios@utexas.edu, UTA.
% edited by Breno Bahia.
% ==================================================================================

       n = params.n;
       m = params.m;
       r = params.r;
       y = params.y;
    A_id = params.A_id;
   At_id = params.At_id;
   alpha = params.alpha;
      mu = params.mu;
     tol = params.BFGD_tol; 

% Initialization of factors 

       W = eye(length(y));
       Ur = randn(m,r);  
       Vr = randn(n,r);  

       Ucur = Ur ;Ucur = Ucur./norm(Ucur, 'fro');
       Vcur = Vr ;Vcur = Vcur./norm(Vcur, 'fro');
       Xcur = Ucur*Vcur';
       Xprev = Xcur;

%  Determination of step-size

       [~, S1, ~] = svds([Ucur; Vcur], 1);
       norm_grad = f_grad(W,Xcur);
       [~, S2, ~] = svds(norm_grad,1); 
       grad0 = f_grad(W,zeros(m,n));
       grad1 = f_grad(W,ones(m,n));
       mu_est = 2*norm(grad0 - grad1, 'fro')/(min(m,n));
       eta = 1/( 2 * mu_est * S1(1,1)^2 + 3*S2(1,1));

   i = 1;

% ---------------------------------------------------

   while (i <= params.iters)

       % Residuals
       Res = y(:) - A_id(Xprev);
           
  % Scale parameter (Eq. 21 in Chen and Sacchi (2016))

       MAD = median(abs(Res-median(Res)));    
       sigma = MAD/0.6745;

       % disp([i, sigma]) 

       % Get weight matrix
       flagW = abs(Res./sigma) <= alpha;
       WB = (1.0-(abs(Res./sigma)./alpha).^2).^2;  
       WB = ones(size(WB));
       W = diag(flagW.*WB);

       % Gradients

       f_gradX = f_grad(W,Xprev);
       
       gradU = f_gradX  * Vcur + mu * Ucur*(Ucur'*Ucur - Vcur'*Vcur);
       gradV = f_gradX' * Ucur + mu * Vcur*(Vcur'*Vcur - Ucur'*Ucur);

       % Factors update

       Ucur = Ucur - eta * gradU;
       Vcur = Vcur - eta * gradV;
       
       % No projection:                   % One could project Xcur onto a space

       Xcur = Ucur*Vcur';                 % of Hankel matrices by doing At_id(A_id(Xcur))
                                          % as to guarantee its structure.
       
       % Update Xprev
       Error = norm(Xcur-Xprev,'fro')/norm(Xprev,'fro');

       Xprev = Xcur;       

      if Error<tol; break; end;
       
       % Update counter
       i = i + 1;
   end

% disp([i-1, params.iters]);
  
   Xhat = Xcur;
   
% =========================================================================
       
