function [OUT]= SeisUnPatchlrz(Patches,nt,Minval,Maxval,PARAM)

%  (Patches,nt,dt,it_WL,it_WO,Mint,min_ix,max_ix,dx,ix_WL,ix_WO,Minx,Maxx)
   % Function to unpatch 2D seismic data.
   % Syntax:
   %        [OUT]= SeisUnPatch2D
   %     (Patches,nt,dt,it_WL,it_WO,Mint,min_ix,max_ix,dx,ix_WL,ix_WO,Minx,Maxx)
   % INPUTS:
   %        Patches - Patches of full seismic section
   %        nt - number of time samples in full section
   %        dt - sampling in time
   %        it_WL - window length in time
   %        it_WO - window overlap in time (give in %)
   %        dx - sampling in space
   %        ix_WL - window length in space
   %        it_WO - window overlap in space (give in %)
   %        min_ix - minimum offset to be used in samples
   %        max_ix - maximum offset to be used in samples
   %        Mint/x - vector with minimum time/space in patches
   %        Maxt/x - vector with maximum time/space in patches
   %
   % OUTPUTS:
   %        OUT - Unpatched (full) seismic section
   %
   % Note: This function is a translation from the SeismicJulia package.
   %*Credits: A. Stanton, F. Carozzi, 2017*
%%

      % Get unpatching parameters
      [dt,it_WL,it_WO,x1min,x1max,ix1_WL,ix1_WO]= getPatchParam(PARAM,num2str(ndims(Patches)-1));
      
      % Patches dimensions
		nx1=x1max-x1min+1;
		

      % Window overlaps in samples
		it_WO=round((it_WO*it_WL)/100);
		ix1_WO=round((ix1_WO*ix1_WL)/100);
		

		% Safe-guard for window bigger than data
		if it_WL > nt
			it_WL=nt;
		end

		if ix1_WL > nx1
			ix1_WL=nx1;
		end
		
		    
	   % Allocate output
	   % TODO
	   % Input can be a pure or full quaternion. Can generalize
	   % this section of the code using str2func. if condition 
	   % will be necessary to differ pure and full quaternions.
      OUT=zeros(nt,nx1);
      if isa(Patches,'quaternion')
         OUT=quaternion(OUT,OUT,OUT);
      end
      
%    disp('Apply Tapering...')

   for ipatch=1:size(Patches,ndims(Patches))

      % Set initial tapers
      tapti=0; taptf=0;
      tapx1i=0; tapx1f=0;
          
      % Extract i-th patch
% 	   d_patch=getPatch(Patches,ipatch);
       d_patch=(Patches(:,:,ipatch));
	   [dims_patch] = size(d_patch);
	   
	   ot_patch = Minval(ipatch,1);

	   min_it_patch = floor(ot_patch/dt)+1;
	   max_it_patch = floor(ot_patch/dt)+dims_patch(1);

		if max_it_patch>nt
			max_it_patch=nt;
		end
	
	   min_ix1_patch=Minval(ipatch,2);
	   max_ix1_patch=Maxval(ipatch,2);     
	  

      if min_it_patch>1
         tapti=it_WO;
      end

      if max_it_patch<nt
         taptf=it_WO;
      end

      if min_ix1_patch>x1min
         tapx1i=ix1_WO;
      end

      if max_ix1_patch<x1max
         tapx1f=ix1_WO;
      end
		
         
      % TODO
      % Use linear indexing to apply taper to a 2D array. This
      % will allow me to set up OUT also as 2D and just sum two
      % matrices without having to have several additional lines
      % for a simples summation. The array will probably require
      % to be reshaped, instead of squeezed, at the end.
      
      d_patch = ApplyTaper(d_patch,	max_it_patch-min_it_patch+1,max_ix1_patch-min_ix1_patch+1,tapti,taptf,tapx1i,tapx1f);
	        
      OUT(min_it_patch:max_it_patch,min_ix1_patch:max_ix1_patch) = OUT(min_it_patch:max_it_patch,...
                                                                       min_ix1_patch:max_ix1_patch)+...
                                                               d_patch(1:length(min_it_patch:max_it_patch),...
                                                                       1:length(min_ix1_patch:max_ix1_patch));

      OUT=squeeze(OUT);

   end
       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function OUT=ApplyTaper(IN,nt,nx1,tapti,taptf,tapx1i,tapx1f)
   % Function to apply taper.
   %
   %
   % Credits: AS and FC, 2017.
%%
   % Check input type
   if isa(IN,'quaternion')
   
      OUT=zerosq(size(IN));

      % For pure quaternions
      if ispure(IN)
         OUT=v(OUT);
      end

   else

      OUT=zeros(size(IN));
   end   
   

   tt=1; tx1=1; tx2=1; tx3=1; tx4=1;
   
    for ix1=1:nx1

         if (ix1>=1 && ix1<=tapx1i)
%         tx1 = 1 - cos(((ix1-1)/tapx1i)*pi/2);
          tx1=1/(tapx1i-1)*(ix1-1);
         end
      
        if (ix1>=tapx1i && ix1<=nx1-tapx1f) 
         tx1=1;
        end
      
        if (ix1>nx1-tapx1f && ix1<=nx1)
%         tx1 = cos(((ix1-1-nx1+tapx1f)/tapx1f)*pi/2);
			 tx1=1-1./(tapx1f-1)*(ix1-1-nx1+tapx1f);
        end
        
        for it=1:nt            
               if (it>=1 && it<=tapti)
                  tt = 1.0/(tapti-1)*(it-1);
               end
                        
               if (it>tapti && it<=nt-taptf)
                  tt = 1;
               end
         			      
               if (it>nt-taptf && it<=nt)
                  tt = 1-1.0/(taptf-1)*(it-1-nt+taptf);
		       end
		         OUT(it,ix1) =  tt*tx1*IN(it,ix1);  
         end
    end

